<?php

namespace skin\admin;

use Stringable;

// login page
// ----------

function login($ctx) {
$html = <<<HTML
<form action="/admin/login/" method="post" class="form-layout-h">
    <input type="hidden" name="token" value="{$ctx->csrf('adminlogin')}" />
    
    <div class="form-field-wrap clearfix">
        <div class="form-field-label">{$ctx->lang('admin_login')}:</div>
        <div class="form-field">
            <input class="form-field-input" type="text" name="login" size="50" />
        </div>
    </div>
    
    <div class="form-field-wrap clearfix">
        <div class="form-field-label">{$ctx->lang('admin_password')}:</div>
        <div class="form-field">
            <input class="form-field-input" type="password" name="password" size="50" />
        </div>
    </div>
    
    <div class="form-field-wrap clearfix">
        <div class="form-field-label"></div>
        <div class="form-field">
            <button type="submit">{$ctx->lang('sign_in')}</button>
        </div>
    </div>
</form>
HTML;

$js = <<<JS
ge('as_password').focus();
JS;

return [$html, $js];
}


// index page
// ----------

function index($ctx, $admin_login) {
    return <<<HTML
<div class="admin-page">
    Authorized as <b>{$admin_login}</b><br>
    <!--<a href="/admin/log/">Log</a><br/>-->
    <a href="/admin/uploads/">Uploads</a><br>
    <a href="/admin/logout/?token={$ctx->csrf('logout')}">Sign out</a>
</div>
HTML;
}


// uploads page
// ------------

function uploads($ctx, $uploads, $error) {
return <<<HTML
{$ctx->if_true($error, $ctx->formError, $error)}

{$ctx->bc([
    ['text' => $ctx->lang('admin_title'), 'url' => '/admin/'],
    ['text' => $ctx->lang('blog_uploads')],
])}

<div class="blog-upload-form">
    <form action="/admin/uploads/" method="post" enctype="multipart/form-data" class="form-layout-h">
        <input type="hidden" name="token" value="{$ctx->csrf('addupl')}" />
        
        <div class="form-field-wrap clearfix">
            <div class="form-field-label">{$ctx->lang('blog_upload_form_file')}:</div>
            <div class="form-field">
                <input type="file" name="files[]" multiple>
            </div>
        </div>

        <div class="form-field-wrap clearfix">
            <div class="form-field-label">{$ctx->lang('blog_upload_form_custom_name')}:</div>
            <div class="form-field">
                <input type="text" name="name">
            </div>
        </div>

        <div class="form-field-wrap clearfix">
            <div class="form-field-label">{$ctx->lang('blog_upload_form_note')}:</div>
            <div class="form-field">
                <input type="text" name="note" size="55">
            </div>
        </div>

        <div class="form-field-wrap clearfix">
            <div class="form-field-label"></div>
            <div class="form-field">
                <input type="submit" value="Upload">
            </div>
        </div>
    </form>
</div>

<div class="blog-upload-list">
    {$ctx->for_each($uploads, fn($u) => $ctx->uploadsItem(
        id: $u->id,
        name: $u->name,
        direct_url: $u->getDirectUrl(),
        note: $u->note,
        addslashes_note: $u->note,
        markdown: $u->getMarkdown(),
        size: $u->getSize(), 
    ))}
</div>
HTML;
}

function uploadsItem($ctx, $id, $direct_url, $note, $addslashes_note, $markdown, $name, $size) {
return <<<HTML
<div class="blog-upload-item">
    <div class="blog-upload-item-actions">
        <a href="javascript:void(0)" onclick="var mdel = ge('upload{$id}_md'); mdel.style.display = (mdel.style.display === 'none' ? 'block' : 'none')">{$ctx->lang('blog_upload_show_md')}</a>
        | <a href="javascript:void(0)" onclick="BlogUploadList.submitNoteEdit('/admin/uploads/edit_note/{$id}/?token={$ctx->csrf('editupl'.$id)}', prompt('Note:', '{$addslashes_note}'))">Edit note</a>
        | <a href="/admin/uploads/delete/{$id}/?token={$ctx->csrf('delupl'.$id)}" onclick="return confirm('{$ctx->lang('blog_upload_delete_confirmation')}')">{$ctx->lang('blog_upload_delete')}</a>
    </div>
    <div class="blog-upload-item-name"><a href="{$direct_url}">{$name}</a></div>
    {$ctx->if_true($note, '<div class="blog-upload-item-note">'.$note.'</div>')}
    <div class="blog-upload-item-info">{$size}</div>
    <div class="blog-upload-item-md" id="upload{$id}_md" style="display: none">
        <input type="text" value="{$markdown}" onclick="this.select()" readonly size="30">
    </div>
</div>
HTML;
}

function postForm(\SkinContext $ctx,
                  string|Stringable $title,
                  string|Stringable $text,
                  string|Stringable $short_name,
                  array $langs,
                  array $js_texts,
                  string|Stringable|null $date = null,
                  bool $is_edit = false,
                  ?bool $saved = null,
                  ?bool $visible = null,
                  ?bool $toc = null,
                  string|Stringable|null $post_url = null,
                  ?int $post_id = null,
                  ?string $lang = null): array {

$form_url = !$is_edit ? '/articles/write/' : $post_url.'edit/';

// breadcrumbs
$bc_tree = [
    ['url' => '/articles/?lang='.$lang, 'text' => $ctx->lang('articles')]
];
if ($is_edit) {
    $bc_tree[] = ['url' => $post_url.'?lang='.$lang, 'text' => $ctx->lang('blog_view_post')];
} else {
    $bc_tree[] = ['text' => $ctx->lang('blog_new_post')];
}

$html = <<<HTML
<div class="form-error" id="form-error" style="display:none"></div>
{$ctx->if_true($saved, fn() => '<div class="form-success">'.$ctx->lang('info_saved').'</div>')}
{$ctx->bc($bc_tree, 'padding-bottom: 20px')}
<table cellpadding="0" cellspacing="0" class="blog-write-table">
<tr>
    <td id="form_first_cell">
        <form class="blog-write-form form-layout-v" name="postForm" action="{$form_url}" method="post">
            <div class="form-field-wrap clearfix">
                <div class="form-field-label">{$ctx->lang('blog_write_form_title')}</div>
                <div class="form-field">
                    <input class="form-field-input" type="text" name="title" value="{$title}" />
                </div>
            </div>

            <div class="form-field-wrap clearfix">
                <div class="form-field-label">{$ctx->lang('blog_write_form_text')}</div>
                <div class="form-field">
                    <textarea class="form-field-input" name="text" wrap="soft">{$text}</textarea><br/>
                    <a class="blog-write-form-toggle-link" id="toggle_wrap" href="">{$ctx->lang('blog_write_form_toggle_wrap')}</a>
                </div>
            </div>

            <div class="form-field-wrap clearfix">
                <table class="blog-write-options-table">
                    <tr>
                        <td>
                            <div class="clearfix">
                                <div class="form-field-label">{$ctx->lang('blog_post_options')}</div>
                                <div class="form-field">
                                    <label for="visible_cb"><input type="checkbox" id="visible_cb" name="visible"{$ctx->if_true($visible, ' checked="checked"')}> {$ctx->lang('blog_write_form_visible')}</label>
                                </div>
                            </div>
                        </td>
                        <td>
                            <div class="clearfix">
                                <div class="form-field-label">{$ctx->lang('blog_text_options')}</div>
                                <div class="form-field">
                                    <label for="toc_cb"><input type="checkbox" id="toc_cb" name="toc"{$ctx->if_true($toc, ' checked="checked"')}> {$ctx->lang('blog_write_form_toc')}</label>
                                    &nbsp;<select name="lang">
                                        {$ctx->for_each($langs, fn($l) => '<option value="'.$l->value.'"'.($l->value == $lang ? ' selected="selected"' : '').'>'.$l->value.'</option>')}
                                    </select>
                                </div>
                            </div>
                        </td>
                        <td>
                            <div class="clearfix">
                                <div class="form-field-label">{$ctx->lang('blog_write_form_date')}</div>
                                <div class="form-field">
                                    <input type="date" name="date"{$ctx->if_true($date, ' value="'.$date.'"')}>
                                </div>
                            </div>
                        </td>
                    </tr>
                    <tr>
                        <td colspan="2">
                            <div class="clearfix">
                                <div class="form-field-label">{$ctx->lang('blog_write_form_short_name')}</div>
                                <div class="form-field">
                                    <input class="form-field-input" type="text" name="{$ctx->if_then_else($is_edit, 'new_short_name', 'short_name')}" value="{$short_name}" />
                                </div>
                            </div>
                        </td>
                        <td>
                            <div class="form-field-label">&nbsp;</div>
                            <div class="form-field">
                                <button type="submit" name="submit_btn"><b>{$ctx->lang($is_edit ? 'save' : 'blog_write_form_submit_btn')}</b></button>
                            </div>
                        </td>
                    </tr>
                </table>
            </div>
        </form>
        <div id="form_placeholder"></div>
    </td>
    <td>
        <div class="blog-write-form-preview post_text" id="preview_html"></div>
    </td>
</tr>
</table>
HTML;

$js_params = [
    'langs' => array_map(fn($lang) => $lang->value, $langs),
    'token' => $is_edit ? csrf_get('editpost'.$post_id) : csrf_get('post_add')
];
if ($is_edit)
    $js_params += [
        'edit' => true,
        'id' => $post_id,
        'texts' => $js_texts
    ];
$js_params = jsonEncode($js_params);

$js = <<<JAVASCRIPT
cur.form = new AdminWriteEditForm({$js_params});
JAVASCRIPT;

return [$html, $js];
}


function pageForm($ctx,
                  string|Stringable $title,
                  string|Stringable $text,
                  string|Stringable $short_name,
                  array $langs,
                  bool $is_edit = false,
                  ?bool $saved = null,
                  bool $visible = false,
                  ?array $js_text = null): array {
$form_url = '/'.$short_name.'/'.($is_edit ? 'edit' : 'create').'/';

// breadcrumbs
if ($is_edit) {
    $bc_html = $ctx->bc([
        ['url' => '/'.$short_name.'/', 'text' => $ctx->lang('view_page')]
    ], 'padding-bottom: 20px');
} else {
    $bc_html = '';
}

$html = <<<HTML
<div class="form-error" id="form-error" style="display:none"></div>
{$ctx->if_true($saved, fn() => '<div class="form-success">'.$ctx->lang('info_saved').'</div>')}
{$bc_html}
<table cellpadding="0" cellspacing="0" class="blog-write-table">
<tr>
    <td id="form_first_cell">
        <form class="blog-write-form form-layout-v" name="pageForm" action="{$form_url}" method="post">
            <div class="form-field-wrap clearfix">
                <div class="form-field-label">{$ctx->lang('pages_write_form_title')}</div>
                <div class="form-field">
                    <input class="form-field-input" type="text" name="title" value="{$title}" />
                </div>
            </div>

            <div class="form-field-wrap clearfix">
                <div class="form-field-label">{$ctx->lang('pages_write_form_text')}</div>
                <div class="form-field">
                    <textarea class="form-field-input" name="text" wrap="soft">{$text}</textarea><br/>
                    <a class="blog-write-form-toggle-link" id="toggle_wrap" href="">{$ctx->lang('pages_write_form_toggle_wrap')}</a>
                </div>
            </div>
            
            {$ctx->if_then_else($is_edit,
                fn() => $ctx->pageFormEditOptions($short_name, $visible),
                fn() => $ctx->pageFormAddOptions($short_name))}
            
        </form>
        <div id="form_placeholder"></div>
    </td>
    <td>
        <div class="blog-write-form-preview post_text" id="preview_html"></div>
    </td>
</tr>
</table>
HTML;

$js_params = [
    'pages' => true,
    'edit' => $is_edit,
    'token' => $is_edit ? $ctx->csrf('editpage'.$short_name) : $ctx->csrf('addpage'),
    'langs' => array_map(fn($lang) => $lang->value, $langs), // still needed for draft erasing
];
if ($js_text !== null)
    $js_params['text'] = $js_text;
$js_params = jsonEncode($js_params);

$js = <<<JS
cur.form = new AdminWriteEditForm({$js_params});
JS;

return [$html, $js];
}

function pageFormEditOptions($ctx, $short_name, $visible) {
return <<<HTML
<div class="form-field-wrap clearfix">
    <table class="blog-write-options-table">
    <tr>
        <td>
            <div class="clearfix">
                <div class="form-field-label">{$ctx->lang('pages_write_form_short_name')}</div>
                <div class="form-field">
                    <input class="form-field-input" type="text" name="new_short_name" value="{$short_name}" />
                </div>
            </div>
        </td>
        <td>
            <div class="clearfix">
                <div class="form-field-label">{$ctx->lang('pages_write_form_options')}</div>
                <div class="form-field">
                    <label for="visible_cb"><input type="checkbox" id="visible_cb" name="visible"{$ctx->if_true($visible, ' checked="checked"')}> {$ctx->lang('pages_write_form_visible')}</label>
                </div>
            </div>
        </td>
    </tr>
    <tr>
        <td rowspan="2">
            <button type="submit" name="submit_btn"><b>{$ctx->lang('pages_write_form_submit_btn')}</b></button>
        </td>
    </tr>
    </table>
</div>
HTML;
}

function pageFormAddOptions($ctx, $short_name) {
return <<<HTML
<div class="form-field-wrap clearfix">
    <div class="form-field-label"></div>
    <div class="form-field">
        <button type="submit" name="submit_btn"><b>{$ctx->lang('pages_write_form_submit_btn')}</b></button>
    </div>
</div>
<input name="short_name" value="{$short_name}" type="hidden" />
HTML;
}

function pageNew($ctx, $short_name) {
return <<<HTML
<div class="page">
    <div class="empty">
        <a href="/{$short_name}/create/">{$ctx->lang('pages_create')}</a>
    </div>
</div>
HTML;

}

// misc
function formError($ctx, $error) {
return <<<HTML
<div class="form-error">{$ctx->lang('error')}: {$error}</div>
HTML;
}

function markdownPreview($ctx, $unsafe_html, $title) {
return <<<HTML
<div class="blog-post">
    {$ctx->if_true($title, '<div class="blog-post-title"><h1>'.$title.'</h1></div>')}
    <div class="blog-post-text">{$unsafe_html}</div>
</div>
HTML;

}